@@ -1,3 +1,24 @@
+/*
+ * This File is part of Davix, The IO library for HTTP based protocols
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+*/
+
 #include <davix_internal.hpp>
 
 #include <auth/davixauth.hpp>
@@ -41,10 +62,21 @@ void opensslErrorMapper(const std::string & msg, DavixError** err){
     std::ostringstream ss;
     ERR_error_string_n(e, buff_err, 255);
     ss << msg << " : " << buff_err;
-    DavixError::setupError(err, openssl_scope, c, ss.str());
+
+    std::string friendlyError = ss.str();
+    if(friendlyError.find(":bad decrypt") != std::string::npos) {
+        // Sometimes openssl will report PEM_R_BAD_BASE64_DECODE as error code,
+        // but actually show bad decrypt in the error message. :(
+        // This is a credential password issue, change status code so we
+        // can query the user for the key password
+        c = StatusCode::CredDecryptionError;
+    }
+
+    DavixError::setupError(err, openssl_scope, c, friendlyError);
 }
 
 static int SSL_pem_passwd_cb(char *buffer, int size, int rwflag, void *userdata){
+    (void) rwflag;
     if(userdata != NULL) {
         const char* passwd = static_cast<const char*>(userdata);
         int passwd_len = strlen(passwd);
@@ -79,10 +111,8 @@ ne_ssl_client_cert *SSL_X509_Pem_Read(const std::string & pkeyfile_str, const st
 
     // load cred
     if (BIO_read_filename(in, credfile) <= 0){
-        char reason[512];
-        ERR_error_string_n(ERR_get_error(), reason, sizeof(reason));
         std::ostringstream msg;
-        msg << "impossible to open " << credfile << ": " << reason;
+        msg << "impossible to open " << credfile << ": ";
         opensslErrorMapper(msg.str(), err);
         ERR_clear_error();
         BIO_free(in);
@@ -90,10 +120,8 @@ ne_ssl_client_cert *SSL_X509_Pem_Read(const std::string & pkeyfile_str, const st
 
     }
     if ( (cert = PEM_read_bio_X509(in, NULL, SSL_pem_passwd_cb, (void*) password)) == NULL){
-        char reason[512];
-        ERR_error_string_n(ERR_get_error(), reason, sizeof(reason));
         std::ostringstream msg;
-        msg << "parse PEM credential failed " << credfile << ": " << reason;
+        msg << "parse PEM credential failed " << credfile << ": ";
         opensslErrorMapper(msg.str(), err);
         ERR_clear_error();
         BIO_free(in);
