@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,33 +27,36 @@
 #include <modules/modules_profiles.hpp>
 #include <neon/neonsessionfactory.hpp>
 #include <davix_context_internal.hpp>
-
-
-
-
-static const std::string _version = DAVIX_VERSION "-" DAVIX_VERSION_TAG;
-
+#include <core/RedirectionResolver.hpp>
+#include <set>
+#include <mutex>
 
 namespace Davix{
 
+struct LibPath;
+
+
+static LibPath lib_path;
+
+
+static bool redirCachingDisabled(){
+    return ( getenv("DAVIX_DISABLE_REDIRECT_CACHING") != NULL);
+}
 
 ///  Implementation f the core logic in davix
 struct ContextInternal
 {
-    ContextInternal(NEONSessionFactory * fsess):
-        _fsess(fsess),
-        _s_buff(65536),
-        _timeout(300),
-        _context_flags(0),
+    ContextInternal():
+        _fsess(new NEONSessionFactory()),
+        _redirectionResolver(new RedirectionResolver(!redirCachingDisabled())),
         _hook_list()
     {
+            DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CORE, "libdavix path {}, version: {}", getLibPath(), version());
     }
 
-    ContextInternal(const ContextInternal & orig):
+    ContextInternal(const ContextInternal & orig) :
         _fsess(new NEONSessionFactory()),
-        _s_buff(orig._s_buff),
-        _timeout(orig._timeout),
-        _context_flags(orig._context_flags),
+        _redirectionResolver(new RedirectionResolver(!redirCachingDisabled())),
         _hook_list(orig._hook_list)
     {
     }
@@ -64,14 +68,12 @@ struct ContextInternal
          return _fsess.get();
     }
 
-    void setBufferSize(const dav_size_t value){
-      _s_buff = value;
+    inline RedirectionResolver* getRedirectionResolver() {
+        return _redirectionResolver.get();
     }
 
-    boost::scoped_ptr<NEONSessionFactory>  _fsess;
-    dav_size_t _s_buff;
-    unsigned long _timeout;
-    bool _context_flags;
+    std::unique_ptr<NEONSessionFactory>  _fsess;
+    std::unique_ptr<RedirectionResolver> _redirectionResolver;
     HookList _hook_list;
 };
 
@@ -81,7 +83,7 @@ struct ContextInternal
 
 
 Context::Context() :
-    _intern(new ContextInternal(new NEONSessionFactory()))
+    _intern(new ContextInternal())
 {
 }
 
@@ -115,6 +117,9 @@ bool Context::getSessionCaching() const{
     return _intern->_fsess->getSessionCaching();
 }
 
+void Context::clearCache() {
+  _intern->_fsess.reset(new NEONSessionFactory());
+}
 
 HttpRequest* Context::createRequest(const std::string & url, DavixError** err){
     return new HttpRequest(*this, Uri(url), err);
@@ -131,7 +136,7 @@ void Context::loadModule(const std::string &name){
         loadGridProfile(*this);
         return;
     }
-    DAVIX_LOG(DAVIX_LOG_WARNING, "No module named %s found", name.c_str());
+    DAVIX_SLOG(DAVIX_LOG_WARNING, DAVIX_LOG_CORE, "No module named {} found", name);
 }
 
 HookList & Context::getHookList(){
@@ -142,11 +147,31 @@ NEONSessionFactory & ContextExplorer::SessionFactoryFromContext(Context & c){
     return *static_cast<NEONSessionFactory*>(c._intern->getSessionFactory());
 }
 
+RedirectionResolver & ContextExplorer::RedirectionResolverFromContext(Context &c) {
+    return *c._intern->getRedirectionResolver();
+}
+
+LibPath::LibPath(){
+    Dl_info shared_lib_infos;
+
+    // do an address resolution on a local function
+    // get this resolution to determine davix shared library path at runtime
+    if( dladdr((void*) &version, &shared_lib_infos) != 0){
+        path = shared_lib_infos.dli_fname;
+    }
+
+}
 
 const std::string & version(){
+    static const std::string _version = DAVIX_VERSION_STRING;
     return _version;
 }
 
 
-} // End Davix
 
+const std::string & getLibPath(){
+    return lib_path.path;
+}
+
+
+} // End Davix
