@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,8 +19,6 @@
  *
 */
 
-
-
 #include <utils/davix_logger_internal.hpp>
 
 #include <status/davixstatusrequest.hpp>
@@ -49,13 +48,13 @@ static IOChainContext  getIOContext( Context & context, const Uri & uri, const R
 
 }
 
-static const std::string simple_listing("<propfind xmlns=\"DAV:\"><prop><resourcetype><collection/></resourcetype></prop></propfind>");
-
-static const std::string stat_listing("<?xml version=\"1.0\" encoding=\"utf-8\" ?><D:propfind xmlns:D=\"DAV:\" xmlns:L=\"LCGDM:\"><D:prop>"
-                                      "<D:displayname/><D:getlastmodified/><D:creationdate/><D:getcontentlength/>"
-                                      "<D:resourcetype><D:collection/></D:resourcetype><L:mode/>"
-                                      "</D:prop>"
-                                      "</D:propfind>");
+// static const std::string simple_listing("<propfind xmlns=\"DAV:\"><prop><resourcetype><collection/></resourcetype></prop></propfind>");
+//
+// static const std::string stat_listing("<?xml version=\"1.0\" encoding=\"utf-8\" ?><D:propfind xmlns:D=\"DAV:\" xmlns:L=\"LCGDM:\"><D:prop>"
+//                                       "<D:displayname/><D:getlastmodified/><D:creationdate/><D:getcontentlength/><D:quota-used-bytes/>"
+//                                       "<D:resourcetype><D:collection/></D:resourcetype><L:mode/>"
+//                                       "</D:prop>"
+//                                       "</D:propfind>");
 
 struct Davix_dir_handle{
 
@@ -151,7 +150,7 @@ DavPosix::~DavPosix(){
 }
 
 DAVIX_DIR* internal_opendir(Context & context, const RequestParams* params, const std::string & url){
-    Ptr::Scoped<DAVIX_DIR> dir(new DAVIX_DIR(context, url, params));
+    std::unique_ptr<DAVIX_DIR> dir(new DAVIX_DIR(context, url, params));
     dir->end = ! dir->io_chain.nextSubItem(dir->io_context,dir->start_entry_name, dir->start_entry_st);
     return dir.release();
 }
@@ -340,6 +339,18 @@ int DavPosix::stat64(const RequestParams *params, const std::string & url, StatI
     return -1;
 }
 
+int DavPosix::get_quota(const RequestParams *params, const std::string & url, QuotaInfo *info, DavixError **err) {
+    TRY_DAVIX{
+        if(info == NULL)
+            throw DavixException(davix_scope_meta(), StatusCode::InvalidArgument, "Argument info is NULL");
+
+        File f(*context, url);
+        f.quotaInfo(params, *info);
+        return 0;
+    }CATCH_DAVIX(err)
+    return -1;
+}
+
 int davix_remove_posix(Context* context, const RequestParams * params, const std::string & url, bool directory, DavixError** err){
     DavixError* tmp_err = NULL;
     int ret = -1;
@@ -574,6 +585,7 @@ int DavPosix::close(DAVIX_FD* fd, Davix::DavixError** err){
         if(fd){
             fd->io_handler.resetIO(fd->io_context);
             delete fd;
+            fd = NULL;
         }
     }CATCH_DAVIX(err)
     return 0;
