@@ -24,7 +24,6 @@
 
 #include <utils/davix_logger_internal.hpp>
 #include <utils/davix_gcloud_utils.hpp>
-#include <libs/time_utils.h>
 #include <ne_redirect.h>
 #include <ne_request.h>
 #include <davix_context_internal.hpp>
@@ -32,35 +31,49 @@
 #include <fileops/fileutils.hpp>
 #include <fileops/AzureIO.hpp>
 #include <fileops/S3IO.hpp>
+#include <core/RedirectionResolver.hpp>
 
 
 
 namespace Davix {
 
-class NEONSessionExtended : public NEONSession{
+class NEONSessionWrapper {
 public:
-    NEONSessionExtended(NEONRequest* r, const Uri &uri, const RequestParams &p, DavixError **err)
-        : NEONSession(r->_c, uri, p, err), _r(r)
+    NEONSessionWrapper(NEONRequest* r, const Uri &uri, const RequestParams &p, DavixError **err)
+        : _r(r)
     {
-        if(get_ne_sess() != NULL){
-            ne_hook_pre_send(get_ne_sess(), NEONRequest::neon_hook_pre_send, (void*)r);
-            ne_hook_post_headers(get_ne_sess(), NEONRequest::neon_hook_pre_rec, (void*) r);
+        _sess = ContextExplorer::SessionFactoryFromContext(r->getContext()).provideNEONSession(uri, p, err);
+
+        if(_sess && _sess->get_ne_sess() != NULL){
+            ne_hook_pre_send(_sess->get_ne_sess(), NEONRequest::neon_hook_pre_send, (void*)r);
+            ne_hook_post_headers(_sess->get_ne_sess(), NEONRequest::neon_hook_pre_rec, (void*) r);
         }
     }
 
-    virtual ~NEONSessionExtended(){
-        if(get_ne_sess() != NULL){
-            ne_unhook_pre_send(get_ne_sess(), NEONRequest::neon_hook_pre_send, (void*)_r);
-            ne_unhook_post_headers(get_ne_sess(), NEONRequest::neon_hook_pre_rec, (void*) _r);
+    virtual ~NEONSessionWrapper() {
+        if(_sess->get_ne_sess() != NULL){
+            ne_unhook_pre_send(_sess->get_ne_sess(), NEONRequest::neon_hook_pre_send, (void*)_r);
+            ne_unhook_post_headers(_sess->get_ne_sess(), NEONRequest::neon_hook_pre_rec, (void*) _r);
         }
     }
 
+    ne_session* get_ne_sess() {
+        return _sess->get_ne_sess();
+    }
+
+    bool isRecycledSession() const {
+        return _sess->isRecycledSession();
+    }
+
+    void disable_session_reuse() {
+        _sess->disable_session_reuse();
+    }
+
 private:
+    std::unique_ptr<NEONSession> _sess;
     NEONRequest* _r;
 };
 
-
-
 void configureRequestParamsProto(const Uri &uri, RequestParams &params){
     if(params.getProtocol() == RequestProtocol::Auto){
         const std::string & proto = uri.getProtocol();
@@ -72,11 +85,8 @@ void configureRequestParamsProto(const Uri &uri, RequestParams &params){
             params.setProtocol(RequestProtocol::Gcloud);
         }
     }
-
 }
 
-
-
 void neon_generic_error_mapper(int ne_status, StatusCode::Code & code, std::string & str){
     switch(ne_status){
         case NE_OK:
@@ -144,36 +154,17 @@ void neon_simple_req_code_to_davix_code(int ne_status, ne_session* sess, const s
 
 
 NEONRequest::NEONRequest(HttpRequest & h, Context& context, const Uri & uri_req) :
-    _req_flag(RequestFlag::IdempotentRequest),
-    params(),
+    BackendRequest(context, uri_req),
     _neon_sess(),
     _req(NULL),
-    _current( new Uri(uri_req)),
-    _orig(_current),
     _number_try(0),
     _redirects(0),
     _total_read_size(0),
     _last_read(-1),
-    _vec(),
-    _vec_line(),
-    _content_ptr(),
-    _content_len(0),
-    _content_offset(0),
-    _content_body(),
-    _fd_content(-1),
-    _content_provider(),
-    _ans_size(-1),
-    _expiration_time(),
-    _request_type("GET"),
     _h(h),
-    _c(context),
     req_started(false),
     req_running(false),
-    _last_request_flag(0),
-    _headers_field(),
-    _accepted_202_retries(0),
-    _early_termination(0),
-    _early_termination_error(NULL) {
+    _accepted_202_retries(0) {
 }
 
 
@@ -207,13 +198,13 @@ int NEONRequest::createRequest(DavixError** err){
     }
 
     std::shared_ptr<Uri> redir_url;
-    if(this->params.getTransparentRedirectionSupport()) {
-        redir_url = ContextExplorer::SessionFactoryFromContext(_c).redirectionResolve(_request_type, *_current);
+    if(this->_params.getTransparentRedirectionSupport()) {
+        redir_url = ContextExplorer::RedirectionResolverFromContext(_context).redirectionResolve(_request_type, *_current);
     }
 
     // performing an operation which could change the PFN? Clear all cache entries for selected URL
     if(_request_type == "DELETE" || _request_type == "MOVE") {
-        ContextExplorer::SessionFactoryFromContext(_c).redirectionClean(*_current.get());
+        ContextExplorer::RedirectionResolverFromContext(_context).redirectionClean(*_current.get());
     }
 
     if(redir_url.get() != NULL){
@@ -225,18 +216,18 @@ int NEONRequest::createRequest(DavixError** err){
 
 
     // reconfigure protos
-    configureRequestParamsProto(*_current, params);
+    configureRequestParamsProto(*_current, _params);
 
     // configure S3 params if needed
-    if(params.getProtocol() == RequestProtocol::AwsS3)
+    if(_params.getProtocol() == RequestProtocol::AwsS3)
         configureS3params();
 
     // configure azure params if needed
-    if(params.getProtocol() == RequestProtocol::Azure)
+    if(_params.getProtocol() == RequestProtocol::Azure)
         configureAzureParams();
 
     // configure gcloud params if needed
-    if(params.getProtocol() == RequestProtocol::Gcloud)
+    if(_params.getProtocol() == RequestProtocol::Gcloud)
         configureGcloudParams();
 
     _req= ne_request_create(_neon_sess->get_ne_sess(), _request_type.c_str(), _current->getPathAndQuery().c_str());
@@ -247,7 +238,7 @@ int NEONRequest::createRequest(DavixError** err){
 
 int NEONRequest::instanceSession(DavixError** err){
     DavixError * tmp_err=NULL;
-    _neon_sess.reset(static_cast<NEONSession*>(new NEONSessionExtended(this, *_current, params, &tmp_err)));
+    _neon_sess.reset(new NEONSessionWrapper(this, *_current, _params, &tmp_err));
     if(tmp_err){
         _neon_sess.reset(NULL);
         DavixError::propagateError(err, tmp_err);
@@ -261,16 +252,6 @@ ssize_t NEONRequest::neon_body_content_provider(void* userdata, char* buffer, si
      return (ssize_t) req->_content_provider.callback(req->_content_provider.udata, buffer, buflen);
 }
 
-bool NEONRequest::checkTimeout(DavixError **err){
-    if(_expiration_time.isValid() && _expiration_time < Chrono::Clock(Chrono::Clock::Monolitic).now()){
-        std::ostringstream ss;
-        ss << "timeout of " << params.getOperationTimeout()->tv_sec << "s";
-        DavixError::setupError(err, davix_scope_http_request(), StatusCode::OperationTimeout, ss.str());
-        return true;
-    }
-    return false;
-}
-
 static dav_ssize_t iocontext_content_provider(HttpBodyProvider provider, void* udata, void* buffer, dav_size_t size) {
     return provider(udata, (char*) buffer, size);
 }
@@ -280,7 +261,7 @@ void NEONRequest::configureRequest(){
     // add custom user headers, but make sure they're only added once
     // in case of a redirect
     if(!req_running) {
-        std::copy(params.getHeaders().begin(), params.getHeaders().end(), std::back_inserter(_headers_field));
+        std::copy(_params.getHeaders().begin(), _params.getHeaders().end(), std::back_inserter(_headers_field));
     }
 
     /* Doing a PUT to Azure? Horrific workaround to enforce the odd two-step
@@ -290,7 +271,7 @@ void NEONRequest::configureRequest(){
          _current.get()->queryParamExists("sr") &&
          _current.get()->queryParamExists("sp") && !_current.get()->fragmentParamExists("azuremechanism")) {
 
-        IOChainContext iocontext(_c, *_current, &params);
+        IOChainContext iocontext(_context, *_current, &_params);
 
         using std::placeholders::_1;
         using std::placeholders::_2;
@@ -317,18 +298,14 @@ void NEONRequest::configureRequest(){
     }
 
     // setup timeout
-    if(_expiration_time.isValid() == false
-        && params.getOperationTimeout()->tv_sec != 0){
-        using namespace Chrono;
-        _expiration_time = Clock(Clock::Monolitic).now() + Duration(params.getOperationTimeout()->tv_sec);
-    }
+    setupDeadlineIfUnset();
 
     // setup headers
     for(size_t i=0; i< _headers_field.size(); ++i){
         ne_add_request_header(_req, _headers_field[i].first.c_str(),  _headers_field[i].second.c_str());
     }
     // setup flags
-    ne_set_request_flag(_req, NE_REQFLAG_EXPECT100, params.get100ContinueSupport() &&
+    ne_set_request_flag(_req, NE_REQFLAG_EXPECT100, _params.get100ContinueSupport() &&
                         (_req_flag & RequestFlag::SupportContinue100));
     ne_set_request_flag(_req, NE_REQFLAG_IDEMPOTENT, _req_flag & RequestFlag::IdempotentRequest);
 
@@ -347,33 +324,9 @@ void NEONRequest::configureRequest(){
 }
 
 
-void NEONRequest::configureS3params(){
-    // strange workaround to get S3 compatibility on gcloud to work
-    if(params.getAwsRegion().empty()) {
-        HeaderVec vec = _headers_field;
-        S3::signRequest(params, _request_type, *_current, vec);
-        vec.swap(_headers_field);
-    }
-    else {
-        Uri signed_url = S3::signURI(params, _request_type, *_current, _headers_field, NEON_S3_SIGN_DURATION);
-        _current= std::shared_ptr<Uri>(new Uri(signed_url));
-    }
-}
-
-// TODO: make static?
-void NEONRequest::configureAzureParams(){
-    Uri signed_url = Azure::signURI(params.getAzureKey(), _request_type, *_current, NEON_S3_SIGN_DURATION);
-    _current= std::shared_ptr<Uri>(new Uri(signed_url));
-}
-
-void NEONRequest::configureGcloudParams() {
-    Uri signed_url = gcloud::signURI(params.getGcloudCredentials(), _request_type, *_current, _headers_field, NEON_S3_SIGN_DURATION);
-    _current= std::shared_ptr<Uri>(new Uri(signed_url));
-}
-
 int NEONRequest::processRedirection(int neonCode, DavixError **err){
     int end_status = -1;
-    if (this->params.getTransparentRedirectionSupport()) {
+    if (this->_params.getTransparentRedirectionSupport()) {
         if( neonCode != NE_OK
                 && neonCode != NE_RETRY
                 && neonCode != NE_REDIRECT){
@@ -416,7 +369,7 @@ int NEONRequest::negotiateRequest(DavixError** err){
     std::string ugrpluginid;
 
     const uint64_t s3SizeLimit = (1024ull * 1024ull * 1024ull * 5ull);
-    const int auth_retry_limit = params.getOperationRetry();
+    const int auth_retry_limit = _params.getOperationRetry();
     int code, status, end_status = NE_RETRY;
     _last_read = -1;
     _total_read_size = 0;
@@ -463,7 +416,7 @@ int NEONRequest::negotiateRequest(DavixError** err){
             createError(status, err);
 
             cancelSessionReuse();
-            ContextExplorer::SessionFactoryFromContext(_c).redirectionClean(_request_type, *_orig);
+            ContextExplorer::RedirectionResolverFromContext(_context).redirectionClean(_request_type, *_orig);
             return -1;
         }
 
@@ -476,11 +429,11 @@ int NEONRequest::negotiateRequest(DavixError** err){
                 if(_request_type != "GET") goto default_label;
 
                 _accepted_202_retries++;
-                if(_accepted_202_retries <= params.getAcceptedRetry()) {
+                if(_accepted_202_retries <= _params.getAcceptedRetry()) {
                   std::cerr << "DAVIX: Received 202-Accepted, sleeping for " <<
-                    params.getAcceptedRetryDelay() << " seconds before retrying. (attempt " <<
-                    _accepted_202_retries << " out of " << params.getAcceptedRetry() << ")" << std::endl;
-                  sleep(params.getAcceptedRetryDelay());
+                    _params.getAcceptedRetryDelay() << " seconds before retrying. (attempt " <<
+                    _accepted_202_retries << " out of " << _params.getAcceptedRetry() << ")" << std::endl;
+                  sleep(_params.getAcceptedRetryDelay());
                   endRequest(NULL);
                   return startRequest(err);
                 }
@@ -495,7 +448,7 @@ int NEONRequest::negotiateRequest(DavixError** err){
                    !ugrs3post.empty() && (_content_len >= s3SizeLimit || _current->fragmentParamExists("forceMultiPart")) ) {
                     // Ugly workaround for s3 + multi-part upload + dynafed
                     DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "Initiating dynafed-assisted multi-part upload to S3, posturl: {}, pluginid: {}", ugrs3post, ugrpluginid);
-                    IOChainContext iocontext(_c, *_current, &params);
+                    IOChainContext iocontext(_context, *_current, &_params);
 
                     using std::placeholders::_1;
                     using std::placeholders::_2;
@@ -604,7 +557,7 @@ int NEONRequest::negotiateRequest(DavixError** err){
 //
 bool NEONRequest::requestCleanup(){
     // cleanup redirection
-    ContextExplorer::SessionFactoryFromContext(_c).redirectionClean(_request_type, *_orig);
+    ContextExplorer::RedirectionResolverFromContext(_context).redirectionClean(_request_type, *_orig);
 
     // disable recycling
     // server supporting broken pipelining will trigger if reused
@@ -615,7 +568,7 @@ bool NEONRequest::requestCleanup(){
     if(_current != _orig || _neon_sess->isRecycledSession()){
         DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, " ->  Error when using reycling of session/redirect : cancel and try again");
         // disable session reuse for this request, avoid picking up a session already expired
-        params.setKeepAlive(false);
+        _params.setKeepAlive(false);
         _current = _orig;
         return true;
     }
@@ -637,7 +590,7 @@ int NEONRequest::redirectRequest(DavixError **err){
     old_uri = _current;
     _current= std::shared_ptr<Uri>(new Uri(dst_uri));
     ne_free(dst_uri);
-    ContextExplorer::SessionFactoryFromContext(_c).addRedirection(_request_type, *old_uri, _current);
+    ContextExplorer::RedirectionResolverFromContext(_context).addRedirection(_request_type, *old_uri, _current);
 
 
     // recycle old request and session
@@ -656,7 +609,7 @@ int NEONRequest::redirectRequest(DavixError **err){
 
 int NEONRequest::executeRequest(DavixError** err){
     dav_ssize_t read_status=1, total_read=0;
-    _last_request_flag =0;
+    _internal_status = RequestStatus::kNotStarted;
     _vec.clear();
 
     DAVIX_SCOPE_TRACE(DAVIX_LOG_HTTP, execReq);
@@ -679,7 +632,6 @@ int NEONRequest::executeRequest(DavixError** err){
             }
            total_read += read_status;
         }
-
     }
 
     if(read_status < 0){
@@ -697,17 +649,19 @@ int NEONRequest::executeRequest(DavixError** err){
    if( endRequest(err) < 0){
        return -1;
    }
-    _last_request_flag =1; // 1 -> syn request
-    return 0;
+
+   _internal_status = RequestStatus::kCompletedOneShot;
+   return 0;
 }
 
 int NEONRequest::beginRequest(DavixError** err){
     int ret = -1;
-    _last_request_flag = 0;
+    _internal_status = RequestStatus::kNotStarted;
     _vec.clear();
     if( (ret= startRequest(err)) < 0)
         return -1;
-    _last_request_flag = 2; // 2 -> sequential req
+
+    _internal_status = RequestStatus::kStarted;
     return ret;
 }
 
@@ -759,122 +713,13 @@ dav_ssize_t NEONRequest::readBlock(char* buffer, dav_size_t max_size, DavixError
     DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "NEONRequest::readBlock read {} bytes", read_status);
 
     _total_read_size += read_status;
-    if(params.getTransferMonitorCb()){
+    if(_params.getTransferMonitorCb()){
         dav_ssize_t final_size = getAnswerSize();
-        params.getTransferMonitorCb()(*_current, Transfer::Read, _total_read_size, ((final_size> 0)?(final_size):(0)));
+        _params.getTransferMonitorCb()(*_current, Transfer::Read, _total_read_size, ((final_size> 0)?(final_size):(0)));
     }
     return read_status;
 }
 
-dav_ssize_t NEONRequest::readToFd(int fd, dav_size_t read_size, DavixError** err){
-    dav_ssize_t ret=1, total=0;
-    dav_size_t chunk_size = DAVIX_BLOCK_SIZE;
-    read_size = (read_size==0)?(std::numeric_limits<dav_size_t>::max()):read_size;
-    std::vector<char> buffer(chunk_size);
-
-    while( (ret = readBlock(&buffer[0],
-                            std::min<dav_size_t>(chunk_size,read_size),
-                           err)) >0
-           && ( read_size >0 )){
-           if(((dav_size_t)ret) == chunk_size
-                && chunk_size < DAVIX_MAX_BLOCK_SIZE){ // increase buffer size
-                   chunk_size = std::min<dav_size_t>(chunk_size << 1, DAVIX_MAX_BLOCK_SIZE);
-                   buffer.resize(chunk_size);
-           }
-
-           dav_ssize_t write_len = ret;
-           read_size -= ret;
-           total += ret;
-           do{
-               ret = write(fd, &buffer[0], write_len);
-
-               if (ret == -1 && errno == EINTR) {
-                   continue;
-               } else if (ret < 0) {
-                   DavixError::setupError(err, davix_scope_http_request(),
-                                          StatusCode::SystemError, std::string("Impossible to write to fd").append(strerror(errno)));
-                   return -1;
-               } else {
-                   write_len -= ret;
-               }
-           }while(write_len >0);
-    }
-
-    if(total > 0) return total;
-    return ret;
-}
-
-
-dav_ssize_t NEONRequest::readLine(char* buffer, dav_size_t max_size, DavixError** err){
-    dav_ssize_t ret=-1;
-
-    if( _vec_line.size() > 0){
-       std::vector<char>::iterator it;
-       it = std::find(_vec_line.begin(), _vec_line.end(), '\n');
-
-       if( it  != _vec_line.end()){
-           it ++;
-           const dav_ssize_t read_size = it - _vec_line.begin();
-           std::copy(_vec_line.begin(), it, buffer);
-           _vec_line.erase(_vec_line.begin(), it);
-           return read_size;
-       }
-
-       std::copy(_vec_line.begin(), _vec_line.end(), buffer);
-       const dav_ssize_t n_bytes =  _vec_line.size();
-       _vec_line.clear();
-       ret = readLine(buffer + n_bytes, max_size - n_bytes, err);
-       return (ret >= 0)?(ret + n_bytes):-1;
-    }
-
-    if( ( ret = readSegment(buffer, max_size, true, err)) >= 0){
-        // search for crlf
-        char* p_endline;
-        p_endline = std::find(buffer, buffer+ret, '\n');
-        if( p_endline < buffer+ret) p_endline++;
-        _vec_line.reserve(ret - ( p_endline - buffer ));
-        std::copy(p_endline, buffer + ret, std::back_inserter(_vec_line));
-        *p_endline = '\0';
-        return  p_endline - buffer;
-    }
-    return -1;
-}
-
-
-dav_ssize_t NEONRequest::readSegment(char* p_buff, dav_size_t size_read, bool stop_at_line_boundary, DavixError**err){
-    DavixError* tmp_err=NULL;
-    dav_ssize_t ret, tmp_ret;
-    dav_size_t s_read= size_read;
-    ret = tmp_ret = 0;
-    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "Davix::Request::readSegment: want to read {} bytes ", size_read);
-    bool early_stop = false;
-
-    do{
-        tmp_ret= readBlock(p_buff, s_read, &tmp_err);
-
-        if(tmp_ret > 0 && stop_at_line_boundary && std::find(p_buff, p_buff+tmp_ret, '\n') != p_buff+tmp_ret) {
-            early_stop = true;
-        }
-
-        if(tmp_ret > 0){ // tmp_ret bytes read
-            ret += tmp_ret;
-        }
-
-        if(ret > 0 && ret < (dav_ssize_t) size_read){
-            p_buff+= tmp_ret;
-            s_read -= tmp_ret;
-        }
-
-    }while( !early_stop && tmp_ret > 0
-            &&  ret < (dav_ssize_t) size_read);
-
-    if(tmp_err){
-        DavixError::propagateError(err, tmp_err);
-        return -1;
-    }
-    return ret;
-}
-
 int NEONRequest::endRequest(DavixError** err){
     int status;
     (void) err;
@@ -903,12 +748,9 @@ int NEONRequest::endRequest(DavixError** err){
     return 0;
 }
 
-void NEONRequest::clearAnswerContent(){
-    _vec.clear();
-}
-
-
-
+//------------------------------------------------------------------------------
+// Get response status.
+//------------------------------------------------------------------------------
 int NEONRequest::getRequestCode(){
     if(_early_termination) {
         if(!_early_termination_error) return 200;
@@ -917,60 +759,7 @@ int NEONRequest::getRequestCode(){
     return ne_get_status(_req)->code;
 }
 
-const char* NEONRequest::getAnswerContent(){
-    if(_last_request_flag == 1)
-        return (const char*) &(_vec.at(0));
-    return NULL;
-}
-
-
-std::vector<char> & NEONRequest::getAnswerContentVec(){
-    return _vec;
-}
-
-dav_ssize_t NEONRequest::getAnswerSizeFromHeaders() const{
-    std::string str_file_size="";
-    long size=-1;
-    if( getAnswerHeader(ans_header_content_length, str_file_size)){
-        StrUtil::trim(str_file_size);
-        try{
-            size = toType<long, std::string>()(str_file_size);
-        }catch(...){
-            size = -1;
-        }
-    }
-    if( size == -1){
-       DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "Bad server answer: {} Invalid, impossible to determine answer size", ans_header_content_length);
-    }
-    return static_cast<dav_ssize_t>(size);
-}
-
-dav_ssize_t NEONRequest::getAnswerSize() const{
-    if(_ans_size < 0)
-        _ans_size = getAnswerSizeFromHeaders();
-    return _ans_size;
-}
-
-
-time_t NEONRequest::getLastModified() const{
-    time_t t=0;
-    std::string str_lastmodified;
-    if( getAnswerHeader("Last-Modified", str_lastmodified)){
-        StrUtil::trim(str_lastmodified);
-        try{
-            t = S3::s3TimeConverter(str_lastmodified);
-        }catch(...){
-            str_lastmodified.clear();
-        }
-    }
-    if( str_lastmodified.empty()){
-       DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "Bad server answer: {} Invalid, impossible to determine last modified time");
-    }
-    return t;
-}
-
-
-bool NEONRequest::getAnswerHeader(const std::string &header_name, std::string &value) const{
+bool NEONRequest::getAnswerHeader(const std::string &header_name, std::string &value) const {
     if(_req){
         const char* answer_content = ne_get_response_header(_req, header_name.c_str());
         if(answer_content){
@@ -981,8 +770,7 @@ bool NEONRequest::getAnswerHeader(const std::string &header_name, std::string &v
     return false;
 }
 
-
-size_t NEONRequest::getAnswerHeaders( HeaderVec & vec_headers) const{
+size_t NEONRequest::getAnswerHeaders( HeaderVec & vec_headers) const {
     if(_req){
         void * handle = NULL;
         const char* name=NULL, *value=NULL;
@@ -993,38 +781,6 @@ size_t NEONRequest::getAnswerHeaders( HeaderVec & vec_headers) const{
     return vec_headers.size();
 }
 
-
-
-
-void NEONRequest::setRequestBody(const std::string & body){
-    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "NEONRequest : add request content of size {} ", body.size());
-    _content_body = std::string(body);
-    _content_ptr = (char*) _content_body.c_str();
-    _content_len = strlen(_content_ptr);
-    _fd_content = -1;
-}
-
-void NEONRequest::setRequestBody(const void *buffer, dav_size_t len){
-    _content_ptr = (char*) buffer;
-    _content_len = len;
-    _fd_content = -1;
-}
-
-
-void NEONRequest::setRequestBody(int fd, dav_off_t offset, dav_size_t len){
-    _fd_content = fd;
-    _content_ptr = NULL;
-    _content_len = len;
-    _content_offset = offset;
-}
-
-void NEONRequest::setRequestBody(HttpBodyProvider provider, dav_size_t len, void* udata){
-    _content_len      = len;
-    _content_provider.callback = provider;
-    _content_provider.udata    = udata;
-}
-
-
 void NEONRequest::freeRequest(){
     DavixError::clearError(&_early_termination_error);
     if(_req != NULL){
@@ -1073,7 +829,7 @@ void NEONRequest::neon_hook_pre_send(ne_request *r, void *userdata,
                    ne_buffer *header){
     (void) r;
     NEONRequest* req = (NEONRequest*) userdata;
-    RequestPreSendHook hook = req->_c.getHook<RequestPreSendHook>();
+    RequestPreSendHook hook = req->_context.getHook<RequestPreSendHook>();
     if(hook){
         std::string header_line(header->data, (header->used)-1);
         hook(req->_h, header_line);
@@ -1084,7 +840,7 @@ void NEONRequest::neon_hook_pre_rec(ne_request *r, void *userdata,
                                     const ne_status *status){
     (void) r;
     NEONRequest* req = (NEONRequest*) userdata;
-    RequestPreReceHook hook = req->_c.getHook<RequestPreReceHook>();
+    RequestPreReceHook hook = req->_context.getHook<RequestPreReceHook>();
     if(hook){
         std::ostringstream header_line;
         HeaderVec headers;
