@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,7 +25,6 @@
 
 #include <davix_context_internal.hpp>
 #include <params/davixrequestparams.hpp>
-#include <neon/neonsessionfactory.hpp>
 
 
 #include <pthread.h>
@@ -37,7 +37,7 @@ class NEONSessionFactory;
 class NEONSession
 {
 public:
-    NEONSession(Context & c, const Uri & uri, const RequestParams & p, DavixError** err);
+    NEONSession(NEONSessionFactory &f, const Uri & uri, const RequestParams & p, DavixError** err);
     virtual ~NEONSession();
 
 
@@ -45,8 +45,12 @@ public:
         return _sess;
     }
 
+    inline bool isRecycledSession(){
+        return reused;
+    }
+
     inline void disable_session_reuse(){
-        DAVIX_DEBUG("Disable Session recycling.....");
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_SSL, "Disable Session recycling");
         _session_recycling = false;
     }
 
@@ -56,6 +60,7 @@ private:
     const RequestParams & _params;
     DavixError* _last_error;
     bool _session_recycling;
+    bool reused;
     Uri _u;
 
     NEONSession(const NEONSession &);
@@ -75,7 +80,7 @@ private:
 
 
 void configureSession(ne_session *_sess, const Uri & uri, const RequestParams &params, ne_auth_creds lp_callbac, void* lp_userdata,
-                      ne_ssl_provide_fn cred_callback,  void* cred_userdata);
+                      ne_ssl_provide_fn cred_callback,  void* cred_userdata, bool & reused);
 
 
 }
