@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,12 +23,9 @@
 #define DAVIX_NEONSESSIONFACTORY_H
 
 #include <map>
+#include <mutex>
 #include <utils/davix_uri.hpp>
- 
 #include <neon/neonrequest.hpp>
-#include <boost/thread/locks.hpp>
-#include <boost/thread/mutex.hpp>
-#include <adevpp/containers/cache.hpp>
 
 namespace Davix {
 
@@ -40,15 +38,24 @@ public:
     NEONSessionFactory();
     virtual ~NEONSessionFactory();
 
+    //--------------------------------------------------------------------------
+    // Create a NEONSession.
+    //--------------------------------------------------------------------------
+    std::unique_ptr<NEONSession> provideNEONSession(const Uri &uri, const RequestParams &params, DavixError **err);
+
     /**
       Create a session object or create a recycled  one ( session reuse )
     */
-    int createNeonSession(const RequestParams & params, const Uri & uri, ne_session** sess, DavixError** err);
+    ne_session* createNeonSession(const RequestParams & params, const Uri & uri, DavixError** err);
 
     /**
       store a Neon session object for session reuse purpose
     */
-    int storeNeonSession(ne_session *sess, DavixError** err);
+    void storeNeonSession(ne_session *sess);
+
+    //
+    // opts
+    //
 
     void setSessionCaching(bool caching);
 
@@ -59,26 +66,12 @@ public:
 private:
     // session pool
     std::multimap<std::string, ne_session*> _sess_map;
-    boost::mutex _sess_mut;
-    bool _session_caching, _redir_caching;
+    std::mutex _sess_mut;
+    bool _session_caching;
 
     void internal_release_session_handle(ne_session* sess);
-
     ne_session* create_session(const RequestParams & params, const std::string & protocol, const std::string &host, unsigned int port);
-
     ne_session* create_recycled_session(const RequestParams & params, const std::string & protocol, const std::string &host, unsigned int port);
-
-
-    // redirection pool
-    Adevpp::Cache<std::string, Uri> _redirCache;
-public:
-
-    void addRedirection( const std::string & method, const Uri & origin, boost::shared_ptr<Uri> dest);
-
-    boost::shared_ptr<Uri> redirectionResolve(const std::string & method, const Uri & origin);
-
-    void redirectionClean(const std::string & method, const Uri & origin);
-
 };
 
 void parse_http_neon_url(const std::string & url, std::string & protocol,
