@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,12 +19,13 @@
  *
 */
 
-
 #include <utils/davix_types.hpp>
 #include <davix.hpp>
 #include <davix_internal.hpp>
 #include <string_utils/stringutils.hpp>
 #include "davix_tool_util.hpp"
+#include <utils/davix_logger_internal.hpp>
+#include <utils/davix_gcloud_utils.hpp>
 
 #include <ctype.h>
 #include <simple_getpass/simple_get_pass.h>
@@ -67,7 +69,7 @@ std::string string_from_ptime(const time_t & t){
 
 dav_ssize_t ask_user_login(std::string & login){
     char l[1024] ={0};
-    (std::cout << "Login: ").flush();
+    (std::cerr << "Login: ").flush();
     std::cin.getline(l, 1023);
     login.assign(l);
     std::fill(l, l+1024,'\0');
@@ -75,10 +77,10 @@ dav_ssize_t ask_user_login(std::string & login){
 }
 
 
-dav_ssize_t ask_user_passwd(std::string & passwd){
+dav_ssize_t ask_user_passwd(std::string & passwd, const std::string &prompt = "Password: "){
     char p[1024] ={0};
-    std::cout << "Password: ";
-    std::cout.flush();
+    std::cerr << prompt;
+    std::cerr.flush();
     if(simple_get_pass(p, 1023) > 0){
         passwd.assign(p);
         std::fill(p, p+1024,'\0');
@@ -102,12 +104,48 @@ int configureAuth(OptParams & opts){
     //setup aws creds
     if(opts.aws_auth.first.empty() == false){
         opts.params.setAwsAuthorizationKeys(opts.aws_auth.first, opts.aws_auth.second);
+        opts.params.setAwsRegion(opts.aws_region);
+        opts.params.setAwsToken(opts.aws_token);
+        opts.params.setAwsAlternate(opts.aws_alternate);
         opts.params.setProtocol(RequestProtocol::AwsS3);
     }
 
+    // setup azure creds
+    if(opts.azure_key.empty() == false) {
+        opts.params.setAzureKey(opts.azure_key);
+        opts.params.setProtocol(RequestProtocol::Azure);
+    }
+
+    // setup gcloud creds
+    if(opts.gcloud_creds_path.empty() == false) {
+        gcloud::CredentialProvider provider;
+        opts.params.setGcloudCredentials(provider.fromFile(opts.gcloud_creds_path));
+        opts.params.setProtocol(RequestProtocol::Gcloud);
+    }
+
     return 0;
 }
 
+static bool startswith(const std::string &str, const std::string &prefix) {
+  if(prefix.size() > str.size()) return false;
+
+  for(size_t i = 0; i < prefix.size(); i++) {
+    if(str[i] != prefix[i]) return false;
+  }
+  return true;
+}
+
+const std::string scope_params = "Davix::Tools::Params";
+bool checkProtocolSanity(OptParams &opts, const std::string &url, DavixError **err) {
+    if(!opts.aws_auth.first.empty()) {
+        if(!startswith(url, "s3://") && !startswith(url, "s3s://") && !startswith(url, "http://") && !startswith(url, "https://")) {
+            DavixError::setupError(err, scope_params, StatusCode::InvalidArgument, fmt::format(" S3 credentials cannot be used with the protocol given in this URL : {}", url));
+            return false;
+        }
+    }
+    return true;
+}
+
 
 static void printHookHeaders(char symbol, const std::string & first_msg, const std::string & start_line){
     std::string req_header(start_line);
@@ -209,82 +247,96 @@ std::string mode_to_stringmode(mode_t mode){
     return std::string(res);
 }
 
-
-
-
 int authCallbackLoginPassword(void* userdata, const SessionInfo & info, std::string & login, std::string & password,
                                         int count, DavixError** err){
     (void) info;
     OptParams* opts = static_cast<OptParams*>(userdata);
-    int ret = -1;
-    if(opts->userlogpasswd.first.empty() == false){
+
+    // authentication based on command-line parameters?
+    // never prompt for a password, even if provided credentials are wrong
+    if(opts->userlogpasswd_through_cmdline) {
         login = opts->userlogpasswd.first;
         password = opts->userlogpasswd.second;
-        ret =0;
-    }else {
-        if(count > 0)
-            std::cout << "Authentication Failure, try again:\n";
-        else
-            std::cout << "Authentication needed:\n";
-        if( ask_user_login(login) > 0){
-            if(ask_user_passwd(password) > 0){
-                ret =0;
-            }
+        return 0;
+    }
+
+    // keyboard-based authentication?
+    // prompt again only if last attempt failed
+    if(count == 0 && !opts->userlogpasswd.first.empty()) {
+        login = opts->userlogpasswd.first;
+        password = opts->userlogpasswd.second;
+        return 0;
+    }
+
+    // need to provide credentials through keyboard
+    if(count == 0) {
+        std::cerr << "Basic authentication - server is asking for username and password:\n";
+    }
+    else {
+        std::cerr << "Authentication failure, try again:\n";
+    }
+
+    if(ask_user_login(login) > 0) {
+        if(ask_user_passwd(password) > 0) {
+            std::cerr << std::endl;
+            opts->userlogpasswd.first = login;
+            opts->userlogpasswd.second = password;
+            return 0;
         }
     }
-    std::cout << std::endl;
-    if(ret < 0)
-        DavixError::setupError(err, "Davix::Tool::Auth",
-                               StatusCode::LoginPasswordError,
-                               "No valid login/password provided");
-    return ret;
+
+    DavixError::setupError(err, "Davix::Tool::Auth",
+                           StatusCode::LoginPasswordError,
+                           "No valid login/password provided");
+    return -1;
 }
 
 int authCallbackCert(void* userdata, const SessionInfo & info, X509Credential* cert, DavixError** err){
     (void) info;
     OptParams* opts = static_cast<OptParams*>(userdata);
-    const std::string key_path(opts->priv_key), cred_path(opts->cred_path);
+    std::string key_path(opts->priv_key), cred_path(opts->cred_path);
 
-    if(cred_path.empty() == false){
-        // try without password
-        if(  cert->loadFromFilePEM( ((key_path.empty()== false)?(key_path):(cred_path)),
-                                  cred_path,
-                                  "",
-                                  err) <0){
-            
-            if( (*err)->getStatus() != StatusCode::CredDecryptionError){
-                // credential specific error
-                return -1;
-            }
+    if(key_path.empty()) {
+        key_path = cred_path;
+    }
 
-            // retry with password
-            std::cout << std::endl;
-            DavixError::clearError(err);
-            std::string password;
-            if( ask_user_passwd(password) <0
-                    || cert->loadFromFilePEM(key_path, cred_path, password, err) <0 ){
-                if(err && *err == NULL){
-                    DavixError::setupError(err, "Davix::Tool::Auth",
-                                           StatusCode::CredDecryptionError,
-                                           "Impossible to use and decrypt client credential");
-
-                }
-                return -1;
-             }
+    // empty credentials?
+    if(opts->cred_path.empty()) {
+        if(err && *err == NULL){
+            DavixError::setupError(err, "Davix::Tool::Auth",
+                                   StatusCode::LoginPasswordError,
+                                   "No valid client credential provided");
         }
-        std::cout << std::endl;
+        return -1;
+    }
+
+    // try with existing, cached password first
+    if(cert->loadFromFilePEM(key_path, cred_path, opts->cred_pass, err) <0) {
+        if( (*err)->getStatus() != StatusCode::CredDecryptionError || !opts->cred_pass.empty()) {
+            // credential specific error
+            std::cerr << "("<< (*err)->getErrScope() <<") Error: "<< (*err)->getErrMsg() << std::endl;
+            DavixError::clearError(err);
+            return -1;
+        }
+    }
+    else {
         return 0;
     }
-    if(err && *err == NULL){
-        DavixError::setupError(err, "Davix::Tool::Auth",
-                               StatusCode::LoginPasswordError,
-                               "No valid client credential provided");
+
+    // password is empty, prompt user
+    DavixError::clearError(err);
+    if(ask_user_passwd(opts->cred_pass, "Certificate password: ") < 0
+        || cert->loadFromFilePEM(key_path, cred_path, opts->cred_pass, err) < 0) {
+
+        if(err && *err) {
+            std::cerr << "("<< (*err)->getErrScope() <<") Error: "<< (*err)->getErrMsg() << std::endl;
+        }
+        return -1;
     }
-    return -1;
-
+    std::cerr << std::endl;
+    return 0;
 }
 
-
 std::string string_from_mode(mode_t mode){
     const char* rmask ="xwr";
     std::string str(10,'-');
@@ -324,7 +376,7 @@ std::string SanitiseTildedPath(const char * path){
         std::string newpath(path);
         newpath.erase(0, 1);
         const char * val = std::getenv("HOME");
-        
+
         if(val != NULL){
             newpath.insert(0, val);
         }
@@ -332,7 +384,7 @@ std::string SanitiseTildedPath(const char * path){
             newpath.insert(0, "/");
         }
         return newpath;
-    }   
+    }
     return std::string(path);
 }
 
@@ -412,7 +464,7 @@ void printProgressBar(int out_fd, int percent, dav_ssize_t bytes_transfered, dav
     struct winsize win;
     ioctl(out_fd, TIOCGWINSZ, &win);
 
-    
+
     // w = width of bar, r = 100/w
     double w = static_cast<double>(win.ws_row)/1.5;
     double r = 100/w;
@@ -429,9 +481,9 @@ void printProgressBar(int out_fd, int percent, dav_ssize_t bytes_transfered, dav
 
     std::cout << "\r" "[" << bar << "] ";
     std::cout.width(3);
-    std::cout << percent << "%     " << 
+    std::cout << percent << "%     " <<
         normalize_unit(bytes_transfered) << 'B' << "/" << normalize_unit(size) << 'B';
-    std::cout.width(10); 
+    std::cout.width(10);
     std::cout << normalize_unit(baudrate) << "B/s" <<
         "                     " << std::flush;
 
@@ -455,6 +507,139 @@ int configureMonitorCB(OptParams & opts, Transfer::Type type){
     return 0;
 }
 
+void tokeniseUrl(std::string url, std::deque<std::string>& dirVec){
+    if(url.empty())
+        return;
 
+    std::string::iterator iter;
+
+    for(iter = url.begin(); iter != url.end(); ){
+        std::string::iterator tmpIter = std::find(iter, url.end(), '/');
+        std::string tmp = std::string(iter, tmpIter);
+        dirVec.push_back(tmp);
+
+        iter = tmpIter;
+        if(tmpIter != url.end())
+            ++iter;
+    }
 }
+
+int mkdirP(std::string url, bool trim){
+
+    int ret = -1;
+
+    std::deque<std::string> dirVec;
+    std::string::iterator iter;
+
+    for(iter = url.begin(); iter != url.end(); ){
+        std::string::iterator tmpIter = std::find(iter, url.end(), '/');
+        std::string tmp = std::string(url.begin(), tmpIter);
+        dirVec.push_back(tmp);
+
+        iter = tmpIter;
+        if(tmpIter != url.end())
+            ++iter;
+    }
+
+    ret = mkdirP(dirVec, trim);
+
+    return ret;
 }
+
+int mkdirP(std::deque<std::string>& dirVec, bool trim){
+    std::deque<std::string> dirList;
+
+    // ignore . entries, don't need to create
+    if(dirVec[0] == "." || dirVec[0] == "./")
+        dirVec.pop_front();
+
+    // discard first and last entry, for cases where first already exists and last is a file to be create/open
+    if(trim){
+        dirVec.pop_front();
+        dirVec.pop_back();
+    }
+
+   /*
+    //TODO: optimise logic to skip tokens that have been created previously
+    for(unsigned int i=0; i<dirVec.size(); ++i){
+        if(mkdir(dirVec[i].c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) != 0){
+            if(errno == EEXIST)
+                DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CORE, "Failed to create local directory, {} already exist. Continuing", dirVec[i]);
+            else{
+                std::cout << std::endl << "Failed to create local directory for " << dirVec[i] << std::endl;
+                return -1;
+            }
+        }
+        return 0;
+    }
+    return 0;
+    */
+    if(dirVec.size() == 0)
+        return 0;
+
+    int ret = -1;
+
+    // loop backwards and attempt to create dir with largest depth
+    // break when an existing dir has been found, or one has been created successfully
+    // failed attemps are pushed to queue to retry once we have the location of an existing dir
+    for(int i=dirVec.size()-1; i>=0; --i){
+        ret = mkdir(dirVec[i].c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
+
+        if((ret == 0) || (errno == EEXIST)){
+            break;
+        }
+        else{
+            std::string tmp = dirVec[i];
+            dirList.push_front(tmp);
+        }
+    }
+
+    // at this point we only have to create the dir(s) they do not already exist
+    for(unsigned int ii=0; ii<dirList.size(); ++ii){
+        if(mkdir(dirList[ii].c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) != 0){
+            if(errno == EEXIST)
+                DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CORE, "Failed to create local directory, {} already exist. Continuing", dirList[ii]);
+            else{
+                std::cout << std::endl << "Failed to create local directory for " << dirList[ii] << std::endl;
+                return -1;
+            }
+        }
+    }
+    return 0;
+}
+
+
+void batchTransferMonitor(std::string dirPath, std::string msg, int entryCount, int totalEntry){
+    if(isatty(STDERR_FILENO) !=1)
+        return;
+
+    static bool print_header = false;
+    static Chrono::TimePoint start_time, last_time;
+    Chrono::TimePoint current_time;
+    Chrono::Clock clk(Chrono::Clock::Monolitic, Chrono::Clock::Second);
+
+    current_time = clk.now();
+
+
+    if(print_header == false){
+        last_time = start_time = clk.now();
+        print_header = true;
+    }else{
+        if(current_time < (last_time + Chrono::Duration(1)))
+            return;
+    }
+
+    std::cout << "\r" << msg << " " << dirPath <<
+        "      Files processed: ";
+    std::cout.width(10);
+    std::cout << entryCount;
+
+    if(totalEntry != 0)
+        std::cout << "/" << totalEntry;
+
+    std::cout << "                     " << std::flush;
+}
+
+
+} //Tool
+} //Davix
