@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,7 +33,7 @@ const std::string void_str("");
 
 int davix_uri_parse(const std::string & uri_str, UriPrivate & res);
 int davix_uri_cmp(const UriPrivate &u1, const UriPrivate &u2);
-std::string davix_path_escape(const std::string & str);
+std::string davix_path_escape(const std::string & str, bool escapeSlashes);
 std::string davix_path_unescape(const std::string & str);
 
 struct UriPrivate{
@@ -43,9 +44,9 @@ struct UriPrivate{
         path(),
         host(),
         query(),
+        fragment(),
         port(0),
-        _uri_string(),
-        query_and_path(NULL){}
+        _uri_string() {}
 
     UriPrivate(const UriPrivate & orig):
         code(orig.code),
@@ -54,11 +55,12 @@ struct UriPrivate{
         path(orig.path),
         host(orig.host),
         query(orig.query),
+        fragment(orig.fragment),
         port(orig.port),
-        _uri_string(orig._uri_string),
-        query_and_path(NULL){ }
+        _uri_string(orig._uri_string) {}
+
+    ~UriPrivate(){
 
-    ~UriPrivate() {
     }
 
     void parsing(const std::string & uri_string){
@@ -72,6 +74,49 @@ struct UriPrivate{
         }
     }
 
+    void addParam(const std::string & key, const std::string & value, std::string & target) {
+        if(target.size() == 0) {
+            target = key;
+            target.append("=");
+            target.append(value);
+        }
+        else {
+            target.append("&");
+            target.append(key);
+            target.append("=");
+            target.append(value);
+        }
+        _update_string();
+    }
+
+    void addQueryParam(const std::string & key, const std::string & value) {
+        addParam(key, value, query);
+    }
+
+    void addFragmentParam(const std::string & key, const std::string & value) {
+        addParam(key, value, fragment);
+    }
+
+    void addPathSegment(const std::string &seg) {
+        ensureTrailingSlash();
+        path += seg;
+        _update_string();
+    }
+
+    void ensureTrailingSlash() {
+        if(path.size() == 0 || path[path.size() - 1] != '/') {
+            path += "/";
+            _update_string();
+        }
+    }
+
+    void removeTrailingSlash() {
+        if(path.size() != 0 && path[path.size() - 1] == '/') {
+            path.erase(path.size()-1, 1);
+            _update_string();
+        }
+    }
+
     void clear(){
         proto.clear();
         userinfo.clear();
@@ -82,11 +127,24 @@ struct UriPrivate{
         port = 0;
     }
 
+    void _update_string() {
+        std::ostringstream ss;
+        ss << proto << "://";
+        if(userinfo.size() > 0) ss << "@" << userinfo;
+        ss << host;
+        if(port != 0) ss << ":" << port;
+        ss << path;
+        if(query.size() > 0) ss << "?" << query;
+        if(fragment.size() > 0) ss << "#" << fragment;
+
+        _uri_string = ss.str();
+    }
+
     StatusCode::Code code;
     std::string proto, userinfo, path, host, query, fragment;
     unsigned int port;
     std::string _uri_string;
-    boost::scoped_ptr<std::string> query_and_path;
+    std::unique_ptr<std::string> query_and_path;
 
 };
 
@@ -104,6 +162,36 @@ Uri::Uri(const Uri & uri) :
     d_ptr(new UriPrivate(*(uri.d_ptr))){
 }
 
+void Uri::addQueryParam(const std::string & key, const std::string & value) {
+    d_ptr->addQueryParam(Uri::queryParamEscape(key), Uri::queryParamEscape(value));
+}
+
+
+void Uri::addFragmentParam(const std::string & key, const std::string & value) {
+    d_ptr->addFragmentParam(key, value);
+}
+
+void Uri::addPathSegment(const std::string &seg) {
+    d_ptr->addPathSegment(seg);
+}
+
+void Uri::ensureTrailingSlash() {
+    d_ptr->ensureTrailingSlash();
+}
+
+void Uri::removeTrailingSlash() {
+    d_ptr->removeTrailingSlash();
+}
+
+bool Uri::queryParamExists(const std::string &key) const {
+    ParamVec queryVec = this->getQueryVec();
+    for(ParamVec::iterator it = queryVec.begin(); it != queryVec.end(); it++) {
+        if(it->first == key)
+            return true;
+    }
+    return false;
+}
+
 Uri::~Uri(){
     delete d_ptr;
 }
@@ -134,6 +222,25 @@ const std::string & Uri::getPath() const {
     return d_ptr->path;
 }
 
+void Uri::setPath(const std::string & path) {
+    d_ptr->path = path;
+    d_ptr->_update_string();
+}
+
+void Uri::setProtocol(const std::string & protocol) {
+    d_ptr->proto = protocol;
+    d_ptr->_update_string();
+}
+
+void Uri::httpizeProtocol() {
+    if(d_ptr->proto == "s3" || d_ptr->proto == "dav" || d_ptr->proto == "gcloud") {
+        setProtocol("http");
+    }
+    if(d_ptr->proto == "s3s" || d_ptr->proto == "davs" || d_ptr->proto == "gclouds") {
+        setProtocol("https");
+    }
+}
+
 const std::string & Uri::getPathAndQuery() const {
     if(d_ptr->query_and_path.get() == NULL){
         if(d_ptr->query.size() > 0){
@@ -145,6 +252,56 @@ const std::string & Uri::getPathAndQuery() const {
     return *(d_ptr->query_and_path);
 }
 
+static ParamVec splitParams(const std::string &str) {
+    ParamVec params;
+    const std::string paramSeparator = "&";
+    const std::string pairSeparator = "=";
+
+    std::string::size_type curr = 0;
+    while(curr < str.length()) {
+        std::string::size_type next = str.find(paramSeparator, curr);
+        if(next == std::string::npos) next = str.length();
+        std::string param = str.substr(curr, next-curr);
+
+        // separate the two parts between '='
+        std::string::size_type eq = param.find(pairSeparator);
+        if(eq == std::string::npos) {
+            params.push_back(ParamLine(param, ""));
+        } else {
+            params.push_back(ParamLine(param.substr(0, eq), param.substr(eq+1, param.length())));
+        }
+
+        curr = next+1;
+    }
+    return params;
+}
+
+const std::string & Uri::getFragment() const {
+    return d_ptr->fragment;
+}
+
+bool Uri::fragmentParamExists(const std::string &param) const {
+    ParamVec fragmentVec = splitParams(getFragment());
+    for(ParamVec::iterator it = fragmentVec.begin(); it != fragmentVec.end(); it++) {
+        if(it->first == param)
+            return true;
+    }
+    return false;
+}
+
+const std::string Uri::getFragmentParam(const std::string &param) const {
+    ParamVec fragmentVec = splitParams(getFragment());
+    for(ParamVec::iterator it = fragmentVec.begin(); it != fragmentVec.end(); it++) {
+        if(it->first == param)
+            return it->second;
+    }
+    return "";
+}
+
+ParamVec Uri::getQueryVec() const {
+    return splitParams(getQuery());
+}
+
 const std::string & Uri::getQuery() const{
     return d_ptr->query;
 }
@@ -176,13 +333,34 @@ bool Uri::operator ==(const Uri & u2) const{
 
 
 std::string Uri::escapeString(const std::string & str){
-    return davix_path_escape(str);
+    return davix_path_escape(str, false);
 }
 
 std::string Uri::unescapeString(const std::string & str){
     return davix_path_unescape(str);
 }
 
+std::string Uri::queryParamEscape(const std::string & str) {
+    return davix_path_escape(str, true);
+}
+
+// Determine whether this is a URL, and if so, URI-escape the right part.
+// Otherwise, simple string concatenation.
+std::string Uri::join(const std::string &left, const std::string &right) {
+    Uri leftUri(left);
+
+    if(leftUri.getStatus() != StatusCode::OK) {
+        if(left.size() == 0 || left[left.size()-1] != '/') {
+            return left + "/" + right;
+        }
+        return left + right;
+    }
+
+    leftUri.ensureTrailingSlash();
+    return leftUri.getString() + Uri::escapeString(right);
+}
+
+
 
 // FIX IT : does not manage properly ../
 Uri Uri::fromRelativePath(const Uri &uri, const std::string &relPath){
@@ -399,9 +577,11 @@ int davix_uri_parse(const std::string & uri_str, UriPrivate & res)
             /* No colon; => p = path-abempty */
         } else if (p + 1 != pa) {
             /* => p = colon */
-            res.port = atoi(p + 1);
-            if( res.port  == 0)
-                 res.port  = UINT_MAX;
+            if(*p == ':') {
+              res.port = atoi(p + 1);
+              if( res.port  == 0)
+                res.port  = UINT_MAX;
+            }
         }
         res.host.assign(s, p - s);
 
@@ -507,16 +687,18 @@ std::string davix_path_unescape(const std::string & str)
 
 /* CH must be an unsigned char; evaluates to 1 if CH should be
  * percent-encoded. */
-#define path_escape_ch(ch) (uri_lookup(ch) & URI_ESCAPE)
+inline bool path_escape_ch(const char ch, bool escapeSlashes) {
+    return (uri_lookup(ch) & URI_ESCAPE) || (escapeSlashes && ch == '/');
+}
 
-std::string davix_path_escape(const std::string & str)
+std::string davix_path_escape(const std::string & str, bool escapeSlashes)
 {
     const unsigned char *pnt, *path= (const unsigned char*)(str.c_str());
     char *p;
     size_t count = 0;
 
     for (pnt = (const unsigned char *)path; *pnt != '\0'; pnt++) {
-        count += path_escape_ch(*pnt);
+        count += path_escape_ch(*pnt, escapeSlashes);
     }
 
     if (count == 0) {
@@ -526,9 +708,9 @@ std::string davix_path_escape(const std::string & str)
     char buffer[str.size() + 2 * count + 1];
     p = buffer;
     for (pnt = (const unsigned char *)path; *pnt != '\0'; pnt++) {
-    if (path_escape_ch(*pnt)) {
+    if (path_escape_ch(*pnt, escapeSlashes)) {
         /* Escape it - %<hex><hex> */
-        sprintf(p, "%%%02x", (unsigned char) *pnt);
+        sprintf(p, "%%%02X", (unsigned char) *pnt);
         p += 3;
     } else {
         *p++ = *pnt;
@@ -540,6 +722,10 @@ std::string davix_path_escape(const std::string & str)
 
 #undef path_escape_ch
 
+std::ostream& operator<< (std::ostream& stream, const Davix::Uri & _u){
+    stream << _u.getString();
+    return stream;
+}
 
 } // namespace Davix
 
@@ -548,5 +734,3 @@ std::ostream& operator<< (std::ostream& stream, const Davix::Uri & _u){
 	stream << _u.getString();
 	return stream;
 }
-
-
