@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -71,7 +72,7 @@ struct MetalinkParser::MetalinkParserIntern{
 
     inline int startElem(const std::string & name){
 
-        DAVIX_TRACE("MetalinkParser: <tag> %s", name.c_str());
+        DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_XML, "MetalinkParser: <tag> {}", name.c_str());
         const MetalinkTag::MetalinkParserTag t = getTag(name);
         if( t == MetalinkTag::Invalid){
             return 0;
@@ -89,26 +90,35 @@ struct MetalinkParser::MetalinkParserIntern{
 
     inline int endElem(const std::string & name){
         const MetalinkTag::MetalinkParserTag t = getTag(name);
+        StrUtil::trim(_buffer);
         std::string & replic = _buffer;
 
         // metalink 3.0
         if(matchStack(_tagStack, url_stack, url_stack_size)){
-            DAVIX_TRACE("MetalinkParser 3.0 : Replica URL %s", replic.c_str());
-            _fvec.push_back(File(_c, Uri(trim(replic, StrUtil::isSpace))));
+            DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_XML, "MetalinkParser 3.0 : Replica URL {}", replic.c_str());
+            _fvec.push_back(File(_c, Uri(replic)));
         }
         if(matchStack(_tagStack, size_stack, size_stack_size)){
-            DAVIX_TRACE("MetalinkParser 3.0 : Replica size %d", replic.c_str());
-            _filesize = static_cast<dav_size_t>(strtoul(replic.c_str(), NULL, 10));
+            DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_XML, "MetalinkParser 3.0 : Replica size {}", replic.c_str());
+            try{
+                _filesize = toType<dav_size_t, std::string>()(replic);
+            }catch(...){
+                _filesize =0;
+            }
         }
 
         // metalink 4.0
         if(matchStack(_tagStack, url_stack_meta4, url_stack_size_meta4)){
-            DAVIX_TRACE("MetalinkParser 4.0 : Replica URL %s", replic.c_str());
-            _fvec.push_back(File(_c, Uri(trim(replic, StrUtil::isSpace))));
+            DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_XML, "MetalinkParser 4.0 : Replica URL {}", replic.c_str());
+            _fvec.push_back(File(_c, Uri(replic)));
         }
         if(matchStack(_tagStack, size_stack_meta4, size_stack_size_meta4)){
-            DAVIX_TRACE("MetalinkParser 4.0 : Replica size %d", replic.c_str());
-            _filesize = static_cast<dav_size_t>(strtoul(replic.c_str(), NULL, 10));
+            DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_XML, "MetalinkParser 4.0 : Replica size {}", replic.c_str());
+            try{
+                _filesize = toType<dav_size_t, std::string>()(replic);
+            }catch(...){
+                _filesize =0;
+            }
         }
 
         _buffer.clear();
@@ -146,18 +156,22 @@ dav_size_t MetalinkParser::getSize() const{
 int MetalinkParser::parserStartElemCb(int parent,
                                const char *nspace, const char *name,
                                const char **atts){
+    (void) parent;
+    (void) nspace;
+    (void) atts;
     return d_ptr->startElem(name);
 }
 
 int MetalinkParser::parserCdataCb(int state,
                             const char *cdata, size_t len){
-
+    (void) state;
     return d_ptr->dataElem(cdata, len);
 }
 
 int MetalinkParser::parserEndElemCb(int state,
                             const char *nspace, const char *name){
-
+    (void) state;
+    (void) nspace;
     return d_ptr->endElem(name);
 }
 
